(*************************************************
Selstyle is a combination of 2 custom combo boxes that
can interact with each other.

TPvlColorCombo which allows you to select a color and
TPvlPatternCombo which allows you to select a brushstyle.

TPvlColorCombo Additional Properties :
   PatternCombo : The Patterncombo that it must synchronise  with
   ColorValue: The Color currently selected
   Pattern: The BrushStyle that must be used in drawing the samples
   KeyDropDown : Does the down arrow drop the combo box?
   AllowWhite : Include white in the samples?
TPvlColorCombo Additional Events :
   OnChange : The color has changed.

TPvlPatternCombo Additional Properties :
   ColorCombo : The Colorcombo that it must synchronise  with
   ColorValue: The Color that must be used in drawing the samples
   Pattern: The BrushStyle currently selected
   KeyDropDown : Does the down arrow drop the combo box?
   AllowClear : Include bsClear in the samples?
TPvlPatternCombo Additional Events :
   OnChange : The Pattern has changed.


This Source Code remains Copyrighted to
CopyRight (c) 1997 PA van Lonkhuyzen

It may be distributed/modified/used freely.

The Author may be contacted via
email peterv@global.co.za
post P A van Lonkhuyzen
     PO Box 1041
     Randpark Ridge
     2156
     South Africa
***************************************************)
{$D-,L-} {Remove if you wish to debug this code}
unit selstyle;

interface

uses
  Windows,messages,forms, Classes, Graphics, Controls,  StdCtrls;

type
  TPvlPatternCombo = class;
  TPvlColorCombo = class(TCustomComboBox)
  private
    FLinkWindow : TPvlPatternCombo;
    FColorValue: TColor;
    FPattern: TBrushStyle;
    FOnChange: TNotifyEvent;
    FKeyDropDown,
    FAllowWhite : Boolean;
    procedure SetColor(Value: TColor);
    procedure SetPattern(Value: TBrushStyle);
    procedure SetLinkWindow(Value: TPvlPatternCombo);
    procedure SetAllowWhite(Value : Boolean);
  protected
    procedure wmKeyDown(var Message : TWMKEY); message wm_keydown;
    procedure wmKeyUp(var Message : TWMKEY); message wm_keyUp;
    procedure Loaded; override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Click; override;
    procedure Change; dynamic;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property PatternCombo : TPvlPatternCombo read FLinkWindow Write SetLinkWindow;
    property ColorValue: TColor read FColorValue write SetColor default clBlack;
    property Pattern: TBrushStyle read FPattern write SetPattern default bssolid;
    Property KeyDropDown : Boolean read FKeyDropDown write FKeyDropDown;
    Property AllowWhite : Boolean read FAllowWhite write SetAllowWhite;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property ParentColor;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
{$IFDEF WIN32}
    property OnStartDrag;
{$ENDIF}
  end;
  TPvlPatternCombo = class(TCustomComboBox)
  private
    FKeyDropDown,
    FAllowClear : Boolean;
    FLinkWindow : TPvlColorCombo;
    FColorValue: TColor;
    FPattern: TBrushStyle;
    FOnChange: TNotifyEvent;
    procedure SetColor(Value: TColor);
    procedure SetPattern(Value: TBrushStyle);
    procedure SetLinkWindow(Value: TPvlColorCombo);
    procedure SetAllowClear(Value : Boolean);
  protected
    procedure wmKeyDown(var Message : TWMKEY); message wm_keydown;
    procedure wmKeyUp(var Message : TWMKEY); message wm_keyUp;
    procedure Loaded; override;
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Click; override;
    procedure Change; dynamic;
  public
    constructor Create(AOwner: TComponent); override;
  published
    Property KeyDropDown : Boolean read FKeyDropDown write FKeyDropDown;
    property ColorCombo : TPvlColorCombo read FLinkWindow Write SetLinkWindow;
    property ColorValue: TColor read FColorValue write SetColor default clBlack;
    property Pattern: TBrushStyle read FPattern write SetPattern default bssolid;
    Property AllowClear : Boolean read FAllowClear write SetAllowClear;
    property Color;
    property Ctl3D;
    property DragMode;
    property DragCursor;
    property Enabled;
    property ParentColor;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
{$IFDEF WIN32}
    property OnStartDrag;
{$ENDIF}
  end;

procedure Register;

implementation


const
  ColorCount = 16;
  Colors: array [0..ColorCount-1] of TColor = (
    clBlack, clMaroon, clGreen, clOlive, clNavy, clPurple, clTeal, clGray,
    clSilver, clRed, clLime, clYellow, clBlue, clFuchsia, clAqua, clWhite);
  PatternCount = 8;
  Patterns: array [0..PatternCount-1] of TBrushStyle =
    (bsSolid, bsHorizontal, bsVertical, bsFDiagonal,
     bsBDiagonal, bsCross, bsDiagCross, bsClear);

constructor TPvlColorCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := csOwnerDrawFixed;
  FColorValue := clBlack;
  FPattern:=bsSolid;
end;

procedure TPvlColorCombo.wmKeyUp(var Message : TWMKEY);
var old : integer;
begin
  if fkeyDropDown and (message.charcode=vk_down) and not(longbool(sendmessage(handle,cb_getDroppedstate,0,0))) then
  begin
    old:=itemindex;
    sendmessage(handle,cb_showdropdown,LONGINT(TRUE),0);
    itemindex:=old;
    message.result:=0;
  end
  else
    inherited;
end;

procedure TPvlColorCombo.wmKeyDown(var Message : TWMKEY);
begin
  if fkeyDropDown and (message.charcode=vk_down) and not(longbool(sendmessage(handle,cb_getDroppedstate,0,0))) then
  begin
    message.result:=0;
  end
  else
    inherited;
end;

procedure TPvlColorCombo.SetLinkWindow(Value: TPvlPatternCombo);
begin
  if FLinkWindow<>Value then
  begin
    FLinkWindow:=Value;
    if not (csLoading in ComponentState) then
      if assigned(FLinkWindow) then
        FLinkWindow.Colorvalue:=ColorValue;
  end
end;

procedure TPvlColorCombo.SetAllowWhite(Value : Boolean);
var i,j : Integer;
begin
  if FAllowWhite<>Value then
  begin
    FAllowWhite:=Value;
    j:=1;
    If not FAllowWhite Then
    begin
      if FColorValue=clWhite Then
        FColorValue:=clBlack;
      j:=2;
    end;
    if not (csLoading in ComponentState) then
    begin
      clear;
      for i := 0 to ColorCount-j do
        Items.AddObject('', TObject(Colors[i]));
      Colorvalue:=FColorvalue;
      invalidate;
    end;
  end
end;

procedure TPvlColorCombo.SetColor(Value: TColor);
var
  Item: Integer;
begin
  if (ItemIndex < 0) or (Value <> FColorValue) then
    for Item := 0 to Items.Count-1 do
    begin
      if TColor(Items.Objects[Item]) = Value then
      begin
        FColorValue := Value;
        ItemIndex := Item;
        Change;
        Break;
      end;
    end;
end;

procedure TPvlColorCombo.SetPattern(Value: TBrushStyle);
begin
  if FPattern <> Value then
  begin
    FPattern := Value;
    invalidate;
  end;
end;

procedure TPvlColorCombo.Loaded;
var
  i,j: Integer;
begin
  inherited;
  j:=1;
  If not FAllowWhite Then
  begin
    if FColorValue=clWhite Then
      FColorValue:=clBlack;
    j:=2;
  end;
  for i := 0 to ColorCount-j do
    Items.AddObject('', TObject(Colors[i]));
  ColorValue:=FColorValue;
end;

procedure TPvlColorCombo.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
var
  ARect: TRect;
  OldCol: TColor;
  OldStyle: TBrushStyle;
begin
  ARect := Rect;
  InflateRect(ARect, -2, -2);
  with Canvas do begin
    FillRect(Rect);
    OldCol := Brush.Color;
    OldStyle:= Brush.Style;
    try
      Pen.Color := clBlack;
      Brush.Color := TColor(Items.Objects[Index]);
      Brush.Style:=FPattern;
      Rectangle(ARect.Left, ARect.Top, ARect.Right, ARect.Bottom);
    finally
      Brush.Color := OldCol;
      Brush.Style := OldStyle;
    end;
  end;
end;

procedure TPvlColorCombo.Click;
begin
  if ItemIndex >= 0 then
    ColorValue := TColor(Items.Objects[ItemIndex]);
  inherited Click;
end;

procedure TPvlColorCombo.Change;
begin
  if screen.activecontrol<>self then
    invalidate;
  if assigned(FLinkWindow) Then
    FLinkWindow.Colorvalue:=Colorvalue;
  if Assigned(FOnChange) then FOnChange(Self);
end;

constructor TPvlPatternCombo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := csOwnerDrawFixed;
  FColorValue := clBlack;
  FPattern:=bsSolid;
end;

procedure TPvlPatternCombo.wmKeyUp(var Message : TWMKEY);
var old : integer;
begin
  if fkeyDropDown and (message.charcode=vk_down) and not(longbool(sendmessage(handle,cb_getDroppedstate,0,0))) then
  begin
    old:=itemindex;
    sendmessage(handle,cb_showdropdown,LONGINT(TRUE),0);
    itemindex:=old;
    message.result:=0;
  end
  else
    inherited;
end;

procedure TPvlPatternCombo.wmKeyDown(var Message : TWMKEY);
begin
  if fkeyDropDown and (message.charcode=vk_down) and not(longbool(sendmessage(handle,cb_getDroppedstate,0,0))) then
  begin
    message.result:=0;
  end
  else
    inherited;
end;

procedure TPvlPatternCombo.SetLinkWindow(Value: TPvlColorCombo);
begin
  if FLinkWindow<>Value then
  begin
    FLinkWindow:=Value;
    if not (csLoading in ComponentState) then
      if assigned(FLinkWindow) then
        FLinkWindow.Pattern:=Pattern;
  end
end;

procedure TPvlPatternCombo.SetAllowClear(Value : Boolean);
var i,j : Integer;
begin
  if FAllowClear<>Value then
  begin
    FAllowClear:=Value;
    j:=1;
    If not FAllowClear Then
    begin
      if FPattern=bsClear Then
        FPattern:=bsSolid;
      j:=2;
    end;
    if not (csLoading in ComponentState) then
    begin
      clear;
      for i := 0 to PatternCount-j do
        Items.AddObject('', TObject(Patterns[i]));
      Pattern:=FPattern;
      invalidate;
    end;
  end
end;

procedure TPvlPatternCombo.SetColor(Value: TColor);
begin
  if FColorValue <> Value then
  begin
    FColorValue := Value;
    invalidate;
  end;
end;

procedure TPvlPatternCombo.SetPattern(Value: TBrushStyle);
var
  Item: Integer;
begin
  if (ItemIndex < 0) or (Value <> FPattern) then
    for Item := 0 to Pred(Items.Count) do
    begin
      if TBrushStyle(Items.Objects[Item]) = Value then
      begin
        FPattern := Value;
        ItemIndex := Item;
        Change;
        Break;
      end;
    end;
end;

procedure TPvlPatternCombo.Loaded;
var
  i,j: Integer;
begin
  inherited;
  j:=1;
  If not FAllowClear Then
  begin
    if FPattern=bsClear Then
      FPattern:=bsSolid;
    j:=2;
  end;
  for i := 0 to PatternCount-j do
    Items.AddObject('', TObject(Patterns[i]));
  Pattern:=FPattern;
end;

procedure TPvlPatternCombo.DrawItem(Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
var
  ARect: TRect;
  OldCol: TColor;
  OldStyle: TBrushStyle;
begin
  ARect := Rect;
  InflateRect(ARect, -2, -2);
  with Canvas do begin
    FillRect(Rect);
    OldCol := Brush.Color;
    OldStyle:= Brush.Style;
    try
      Pen.Color := clBlack;
      Brush.Color := FColorValue;
      Brush.Style:=TBrushStyle(Items.Objects[Index]);
      Rectangle(ARect.Left, ARect.Top, ARect.Right, ARect.Bottom);
    finally
      Brush.Color := OldCol;
      Brush.Style := OldStyle;
    end;
  end;
end;

procedure TPvlPatternCombo.Click;
begin
  if ItemIndex >= 0 then Pattern := TBrushStyle(Items.Objects[ItemIndex]);
  inherited Click;
end;

procedure TPvlPatternCombo.Change;
begin
  if screen.activecontrol<>self then
    invalidate;
  if assigned(FLinkWindow) Then
    FLinkWindow.Pattern:=Pattern;
  if Assigned(FOnChange) then FOnChange(Self);
end;

procedure Register;
begin
  RegisterComponents('Peter''s Components', [TPvlPatternCombo,TPvlColorCombo]);
end;

end.
